#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "jheadlinestypes.h"
#include "dialog_pref.h"
#include "interface.h"

static JhPreference *pref = NULL;
static GtkWidget *spb_update_interval;
static GtkWidget *lab_update_interval;
static gboolean expand_fontsize_has_changed = FALSE;

static void
on_spb_expand_fontsize_changed (GtkEditable *editable, gpointer data)
{
  pref->expand_fontsize = gtk_spin_button_get_value_as_int (data);
  expand_fontsize_has_changed = TRUE;
}

static void
on_spb_expand_winwidth_changed (GtkEditable *editable, gpointer data)
{
  pref->expand_window_width = gtk_spin_button_get_value_as_int (data);
}

static void
on_spb_shrink_interval_changed (GtkEditable *editable, gpointer data)
{
  pref->shrink_message_interval = gtk_spin_button_get_value_as_int (data);
}


static void
on_spb_update_interval_changed (GtkEditable *editable, gpointer data)
{
  pref->update_interval = gtk_spin_button_get_value_as_int (data);
  auto_update (FALSE, 0);
}


static void
on_chb_updateauto_toggled (GtkToggleButton *togglebutton, gpointer data)
{
  if (pref->update_automatically)
    {
      pref->update_automatically = FALSE;
      gtk_widget_set_sensitive (lab_update_interval, FALSE);
      gtk_widget_set_sensitive (spb_update_interval, FALSE);
    }
  else
    {
      pref->update_automatically = TRUE;
      gtk_widget_set_sensitive (lab_update_interval, TRUE);
      gtk_widget_set_sensitive (spb_update_interval, TRUE);
    }
}

static void
on_btn_ok_clicked (GtkButton *button, gpointer data)
{
  GtkWidget *dialog = GTK_WIDGET (data);
  gtk_widget_destroy (dialog);

  auto_update (pref->update_automatically, pref->update_interval);

  if (expand_fontsize_has_changed)
    {
      gchar buff[50];

      pref->expand_window_height = pref->expand_fontsize + 2;

      /*    sprintf (buff, "-misc-fixed-*-r-*-*-%d-*-*-*-*-*-*-*", */
      sprintf (buff, "-*-*-*-r-*-*-%d-*-*-*-*-*-*-*", pref->expand_fontsize);

      put_status_info (_("Now allocating fontset..."), NULL);

      if (pref->expand_font != NULL)
	gdk_font_unref (pref->expand_font);
      pref->expand_font = gdk_fontset_load (buff);

      put_status_info ("(status clear)", NULL);
      flash_status_info (_("done"), NULL);

      expand_fontsize_has_changed = FALSE;
    }
}

static GtkWidget *
create_pref_dialog (void)
{
  GtkObject *spb_shrink_interval_adj;
  GtkObject *spb_update_interval_adj;
  GtkObject *spb_expand_winwidth_adj;
  GtkObject *spb_expand_fontsize_adj;
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *table1;
  GtkWidget *spb_shrink_interval;
  GtkWidget *lab_shrink_interval;
  GtkWidget *spb_expand_winwidth;
  GtkWidget *spb_expand_fontsize;
  GtkWidget *lab_expand_winwidth;
  GtkWidget *lab_expand_fontsize;
  GtkWidget *chb_updateauto;
  GtkWidget *dialog_action_area1;
  GtkWidget *btn_ok;

  if (pref == NULL)
    pref = getaddr_pref ();

  dialog1 = gnome_dialog_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);

  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox1), 5);

  table1 = gtk_table_new (5, 2, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table1", table1,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), table1, TRUE, TRUE, 0);

  /*  UPDATE AUTOMATICALLY */
  /* (update automatically) Check button  */
  chb_updateauto =
    gtk_check_button_new_with_label (_("Update automatically"));
  gtk_widget_ref (chb_updateauto);
  gtk_object_set_data_full (GTK_OBJECT (dialog1),
			    "chb_updateauto", chb_updateauto,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (chb_updateauto);
  gtk_table_attach (GTK_TABLE (table1), chb_updateauto, 0, 2, 0, 1,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  if (pref->update_automatically)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chb_updateauto), TRUE);

  /*  UPDATE INTERVAL */
  /* (update interval) label */
  lab_update_interval = gtk_label_new (_("Update interval (min.)"));
  gtk_widget_ref (lab_update_interval);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "lab_update_interval",
			    lab_update_interval,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lab_update_interval);
  gtk_table_attach (GTK_TABLE (table1), lab_update_interval, 0, 1, 1, 2,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lab_update_interval), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (lab_update_interval), 15, 0);

  /* (update interval) spinbutton */
  spb_update_interval_adj = gtk_adjustment_new (pref->update_interval,
						1, 180, 1, 10, 0);
  spb_update_interval =
    gtk_spin_button_new (GTK_ADJUSTMENT (spb_update_interval_adj), 1, 0);
  gtk_widget_ref (spb_update_interval);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spb_update_interval",
			    spb_update_interval,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spb_update_interval);
  gtk_table_attach (GTK_TABLE (table1), spb_update_interval, 1, 2, 1, 2,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);

  /*  SHRINK INTERVAL */
  /* (shrink_interval) label */
  lab_shrink_interval = gtk_label_new (_("Shrinked message interval (sec.)"));
  gtk_widget_ref (lab_shrink_interval);
  gtk_object_set_data_full (GTK_OBJECT (dialog1),
			    "lab_shrink_interval", lab_shrink_interval,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lab_shrink_interval);
  gtk_table_attach (GTK_TABLE (table1), lab_shrink_interval, 0, 1, 2, 3,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lab_shrink_interval), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (lab_shrink_interval), 5, 0);

  /* (shrink_interval) spinbutton */
  spb_shrink_interval_adj = gtk_adjustment_new (pref->shrink_message_interval,
						1, 60, 1, 10, 0);
  spb_shrink_interval =
    gtk_spin_button_new (GTK_ADJUSTMENT (spb_shrink_interval_adj), 1, 0);
  gtk_widget_ref (spb_shrink_interval);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spb_shrink_interval",
			    spb_shrink_interval,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spb_shrink_interval);
  gtk_table_attach (GTK_TABLE (table1), spb_shrink_interval, 1, 2, 2, 3,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);

  /*  EXPAND WINDOW WIDTH */
  /* (expand window width) label */
  lab_expand_winwidth = gtk_label_new (_("Expanded window width"));
  gtk_widget_ref (lab_expand_winwidth);
  gtk_object_set_data_full (GTK_OBJECT (dialog1),
			    "lab_expand_winwidth", lab_expand_winwidth,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lab_expand_winwidth);
  gtk_table_attach (GTK_TABLE (table1), lab_expand_winwidth, 0, 1, 3, 4,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lab_expand_winwidth), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (lab_expand_winwidth), 5, 0);

  /* (expand window width) spinbutton */
  spb_expand_winwidth_adj = gtk_adjustment_new (pref->expand_window_width,
						100, 2000, 1, 10, 0);
  spb_expand_winwidth =
    gtk_spin_button_new (GTK_ADJUSTMENT (spb_expand_winwidth_adj), 1, 0);
  gtk_widget_ref (spb_expand_winwidth);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spb_expand_winwidth",
			    spb_expand_winwidth,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (spb_expand_winwidth, 60, 0);
  gtk_widget_show (spb_expand_winwidth);
  gtk_table_attach (GTK_TABLE (table1), spb_expand_winwidth, 1, 2, 3, 4,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);

  /*  EXPAND FONTSIZE */
  /* (expand fontsize) label */
  lab_expand_fontsize = gtk_label_new (_("Expanded font size"));
  gtk_widget_ref (lab_expand_fontsize);
  gtk_object_set_data_full (GTK_OBJECT (dialog1),
			    "lab_expand_fontsize", lab_expand_fontsize,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lab_expand_fontsize);
  gtk_table_attach (GTK_TABLE (table1), lab_expand_fontsize, 0, 1, 4, 5,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lab_expand_fontsize), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (lab_expand_fontsize), 5, 0);

  /* (expand fontsize) spinbutton */
  spb_expand_fontsize_adj = gtk_adjustment_new (pref->expand_fontsize,
						14, 100, 1, 10, 0);
  spb_expand_fontsize =
    gtk_spin_button_new (GTK_ADJUSTMENT (spb_expand_fontsize_adj), 1, 0);
  gtk_widget_ref (spb_expand_fontsize);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spb_expand_fontsize",
			    spb_expand_fontsize,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (spb_expand_fontsize, 60, 0);
  gtk_widget_show (spb_expand_fontsize);
  gtk_table_attach (GTK_TABLE (table1), spb_expand_fontsize, 1, 2, 4, 5,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);

  /* DIALOG ACTION AREA */
  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1",
		       dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1),
			     GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  btn_ok = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (btn_ok);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "btn_ok", btn_ok,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (btn_ok);
  GTK_WIDGET_SET_FLAGS (btn_ok, GTK_CAN_DEFAULT);

  /* CALLBACKS */
  gtk_signal_connect (GTK_OBJECT (chb_updateauto), "toggled",
		      GTK_SIGNAL_FUNC (on_chb_updateauto_toggled), NULL);
  gtk_signal_connect (GTK_OBJECT (spb_update_interval), "changed",
		      GTK_SIGNAL_FUNC (on_spb_update_interval_changed),
		      spb_update_interval);
  gtk_signal_connect (GTK_OBJECT (spb_shrink_interval), "changed",
		      GTK_SIGNAL_FUNC (on_spb_shrink_interval_changed),
		      spb_shrink_interval);
  gtk_signal_connect (GTK_OBJECT (spb_expand_winwidth), "changed",
		      GTK_SIGNAL_FUNC (on_spb_expand_winwidth_changed),
		      spb_expand_winwidth);
  gtk_signal_connect (GTK_OBJECT (spb_expand_fontsize), "changed",
		      GTK_SIGNAL_FUNC (on_spb_expand_fontsize_changed),
		      spb_expand_fontsize);

  gtk_signal_connect (GTK_OBJECT (btn_ok), "clicked",
		      GTK_SIGNAL_FUNC (on_btn_ok_clicked), dialog1);

  if (!pref->update_automatically)
    {
      gtk_widget_set_sensitive (lab_update_interval, FALSE);
      gtk_widget_set_sensitive (spb_update_interval, FALSE);
    }

  return dialog1;
}

void
on_preferences_activate (GtkMenuItem *menuitem, gpointer data)
{
  GtkWidget *pref_dialog;
  pref_dialog = create_pref_dialog ();
  gtk_widget_show (pref_dialog);

  gtk_grab_add (pref_dialog);
}
