#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "jheadlinestypes.h"
#include "expand.h"
#include "interface.h"

static JhPreference *pref = NULL;
static GtkWidget *ewin;
static gint ptimer = 0;
static GdkPixmap *pmap;
static GdkGC *gc = NULL;
static GtkWidget *darea;

static gint moving_pitch = 5;
static gboolean force_next_headline = FALSE;

static void
destroy_ewin (void)
{
  if (ptimer) {
    gtk_timeout_remove (ptimer);
    ptimer = 0;
  }

  gtk_widget_destroy (ewin);
  main_window_control ("SHOW");
}

static void
ewin_key_press (GtkWidget *widget, GdkEventKey *event)
{
  switch (event->keyval) {

    case GDK_Escape:
    case GDK_q:
      destroy_ewin ();
      break;

    case GDK_space:
    case GDK_Tab:
      force_next_headline = TRUE;
      break;
    }
}

static gboolean
on_ewin_expose_event (GtkWidget *widget, GdkEventExpose *event)
{
  if (pmap)
    gdk_draw_pixmap (widget->window,
		     widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		     pmap,
		     event->area.x, event->area.y,
		     event->area.x, event->area.y,
		     event->area.width, event->area.height);
  return TRUE;
}

static gboolean
on_ewin_configure_event (GtkWidget *widget, GdkEventConfigure *event)
{
  if (pmap)
    gdk_pixmap_unref (pmap);
  pmap = gdk_pixmap_new (widget->window,
			 widget->allocation.width,
			 widget->allocation.height, -1);
  gdk_draw_rectangle (pmap, widget->style->bg_gc[GTK_WIDGET_STATE (widget)],
		      TRUE, 0, 0,
		      widget->allocation.width, widget->allocation.height);

  return TRUE;
}

static gboolean
on_ewin_button_press_event (GtkWidget *widget, GdkEventButton *event)
{
  switch (event->button) {
  case 1:
    force_next_headline = TRUE;
    break;

  case 2:
    moving_pitch = 15;
    break;

  case 3:
    moving_pitch = 1;
    break;

  default:
    ;
  }

  return TRUE;
}

static gboolean
on_ewin_button_release_event (GtkWidget *widget, GdkEventButton *event)
{
  switch (event->button) {
  case 1:
  case 2:
  case 3:
    moving_pitch = 5;
    break;

  default:
    ;
  }

  return TRUE;
}

static gboolean
ewin_scroll_message (JhTarget *target)
{
  static gchar buff[MAXBUFF_LINE] =
    N_("Press button-1 to skip headline message.");
  static gint current = -1;
  static gint xoffset = INIT_EXPAND_WINDOW_WIDTH * EXPAND_SCROL_MSG_START;

  gdk_draw_rectangle (pmap, darea->style->bg_gc[GTK_WIDGET_STATE (darea)],
		      TRUE, 0, 0,
		      darea->allocation.width, darea->allocation.height);

  gdk_draw_text (pmap, pref->expand_font, gc,
		 xoffset, pref->expand_window_height
		 - gdk_string_height (pref->expand_font, buff) / 8,
		 buff, strlen (buff));

  xoffset -= moving_pitch;

  if (xoffset < -gdk_string_width (pref->expand_font, buff)
      + pref->expand_window_width * EXPAND_SCROL_MSG_END ||
      force_next_headline) {

    xoffset = pref->expand_window_width * EXPAND_SCROL_MSG_START;
    current++;

    if (!(target + current)->url)
      current = 0;

    sprintf (buff, "%s: %s", (target + current)->url,
	     gtk_entry_get_text (GTK_ENTRY ((target + current)->entry)));

    force_next_headline = FALSE;
  }

  gdk_draw_pixmap (darea->window,
		   darea->style->fg_gc[GTK_WIDGET_STATE (darea)],
		   pmap, 0, 0, 0, 0,
		   darea->allocation.width, darea->allocation.height);
  return TRUE;
}

void
on_expand_activate (GtkWidget *widget, JhTarget *target)
{
  if (pref == NULL)
    pref = getaddr_pref ();
  
  main_window_control ("HIDE");
  
  /* ewin: expanded window */
  ewin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (ewin), _("jheadlines"));
  gtk_window_set_policy (GTK_WINDOW (ewin), FALSE, FALSE, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (ewin), 5);
  gtk_window_set_position (GTK_WINDOW (ewin), GTK_WIN_POS_MOUSE);

  /* drawing_area: for headline display */
  darea = gtk_drawing_area_new ();
  gtk_drawing_area_size (GTK_DRAWING_AREA (darea),
			 pref->expand_window_width,
			 pref->expand_window_height);
  gtk_container_add (GTK_CONTAINER (ewin), darea);

  gtk_signal_connect (GTK_OBJECT (ewin), "delete_event",
		      GTK_SIGNAL_FUNC (destroy_ewin), NULL);
  gtk_signal_connect (GTK_OBJECT (ewin), "destroy",
		      GTK_SIGNAL_FUNC (destroy_ewin), NULL);
  gtk_signal_connect (GTK_OBJECT (ewin), "key_press_event",
		      GTK_SIGNAL_FUNC (ewin_key_press), NULL);
  gtk_signal_connect (GTK_OBJECT (darea), "expose_event",
		      GTK_SIGNAL_FUNC (on_ewin_expose_event), NULL);
  gtk_signal_connect (GTK_OBJECT (darea), "configure_event",
		      GTK_SIGNAL_FUNC (on_ewin_configure_event), NULL);
  gtk_signal_connect (GTK_OBJECT (darea), "button_press_event",
		      GTK_SIGNAL_FUNC (on_ewin_button_press_event), NULL);
  gtk_signal_connect (GTK_OBJECT (darea), "button_release_event",
		      GTK_SIGNAL_FUNC (on_ewin_button_release_event), NULL);
  gtk_signal_connect (GTK_OBJECT (ewin), "realize",
		      GTK_SIGNAL_FUNC (set_window_icon), NULL);
  gtk_widget_set_events (ewin, GDK_KEY_PRESS_MASK);
  gtk_widget_set_events (darea, GDK_EXPOSURE_MASK |
			 GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);

  gtk_widget_show_all (ewin);

  if (pref->expand_font == NULL) {
    gchar buff[50];

  /*sprintf (buff, "-misc-fixed-*-r-*-*-%d-*-*-*-*-*-*-*", */
    sprintf (buff, "-*-*-*-r-*-*-%d-*-*-*-*-*-*-*",
	     pref->expand_fontsize);
    pref->expand_font = gdk_fontset_load (buff);
  }

  if (gc == NULL)
    gc = gdk_gc_new (ewin->window);

  ptimer = gtk_timeout_add (100, (GtkFunction) ewin_scroll_message, target);
}
