#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <bonobo.h>
#include <gnome.h>

#include "jheadlinestypes.h"
#include "callbacks.h"
#include "interface.h"
#include "dialog_about.h"
#include "dialog_pref.h"
#include "network.h"
#include "shrink.h"
#include "expand.h"
#include "support.h"

#include "pixmaps/window_icon.xpm"

static JhPreference pref;
static GtkWidget *app;
static GtkWidget *appbar;

static JhTarget target[] = {

  {"http://www.asahi.com/", NULL, NULL, NULL},
  {"http://mainichi.jp/", NULL, NULL, NULL},
  {"http://www.nikkei.com/", NULL, NULL, NULL},
  {"http://www.yomiuri.co.jp/", NULL, NULL, NULL},
  {"http://www.sankei.com/", NULL, NULL, NULL},
  {"http://www.tokyo-np.co.jp/", NULL, NULL, NULL},
  {"http://www.jiji.com/", NULL, NULL, NULL},
  {"http://www.cnn.co.jp/", NULL, NULL, NULL},
  {"http://jp.reuters.com/", NULL, NULL},
  {"http://news.livedoor.com/", NULL, NULL},
  {"http://news.yahoo.co.jp/", NULL, NULL},
  {"http://news.infoseek.co.jp/", NULL, NULL},
  {"http://www.j-cast.com/", NULL, NULL},
  {"http://www.daily.co.jp/", NULL, NULL},
  {"http://www.nikkansports.com/", NULL, NULL},
  {"http://www.itmedia.co.jp/news/", NULL, NULL},
  {"http://ascii.jp/", NULL, NULL},
  {"http://japan.cnet.com/", NULL, NULL},
  {"http://rocketnews24.com/", NULL, NULL},

  { NULL, NULL, NULL } /* must null terminated. */
};

static GnomeUIInfo control_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("Update headline news"),
    NULL, (gpointer) on_update_headlines_activate, target, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 'U', (GdkModifierType) GDK_CONTROL_MASK,
    NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
   GNOME_APP_UI_ITEM, N_("Shrink main window"),
   NULL, (gpointer) on_shrink_activate, target, NULL,
   GNOME_APP_PIXMAP_NONE, NULL, 'S', (GdkModifierType) GDK_CONTROL_MASK,
   NULL},
  {
   GNOME_APP_UI_ITEM, N_("Expand headline"),
   NULL, (gpointer) on_expand_activate, target, NULL,
   GNOME_APP_PIXMAP_NONE, NULL, 'E', (GdkModifierType) GDK_CONTROL_MASK,
   NULL},
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_preferences_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (on_quit_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_ABOUT_ITEM (on_about_activate, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] =
{
  {
   GNOME_APP_UI_SUBTREE, N_("Control"),
   NULL,
   control_menu_uiinfo, NULL, NULL,
   GNOME_APP_PIXMAP_NONE, NULL,
   0, (GdkModifierType) NULL, NULL},
  GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
  GNOMEUIINFO_END
};

void
set_window_icon (GtkWidget *w)
{
  GtkStyle *style = gtk_widget_get_style (w);
  GdkPixmap *icon = gdk_pixmap_create_from_xpm_d (w->window,
						  NULL,
						  &style->bg[GTK_STATE_NORMAL],
						  (gchar **) window_icon);

  gdk_window_set_icon (w->window, NULL, icon, NULL);
}

void
jheadlines_init (void)
{
  pref.update_automatically = INIT_UPDATE_AUTOMATICALLY;
  pref.update_interval = INIT_UPDATE_INTERVAL;
  pref.shrink_message_interval = INIT_SHRINK_MESSAGE_INTERVAL;
  pref.expand_window_width = INIT_EXPAND_WINDOW_WIDTH;
  pref.expand_window_height = INIT_EXPAND_WINDOW_HEIGHT;
  pref.expand_fontsize = INIT_EXPAND_FONTSIZE;
}

GtkWidget*
create_app (void)
{
  JhTarget *ptarget = target;
  GtkWidget *toolbar;
  GtkWidget *button_refresh, *button_preference, *button_quit;
  GtkWidget *button_shrink, *button_expand;
  GtkWidget *vbox, *hbox;
  GtkWidget *tmp_toolbar_icon;

  app = gnome_app_new ("Jheadlines", NULL);

  gnome_app_create_menus (GNOME_APP (app), menubar_uiinfo);

  /* TOOLBAR */
  toolbar = gtk_toolbar_new ();
  gtk_widget_show (toolbar);
  gnome_app_add_toolbar (GNOME_APP (app), GTK_TOOLBAR (toolbar), "toolbar",
			 BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,
			 BONOBO_DOCK_TOP, 1, 0, 0);
  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH);

  /* TOOLBAR/REFRESH BUTTON */
  button_refresh = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
					     "gtk-refresh",
					     _("Refresh"),
					     NULL, NULL, NULL, -1);
  gtk_widget_show (button_refresh);
  gtk_signal_connect (GTK_OBJECT (button_refresh), "clicked",
		      GTK_SIGNAL_FUNC (on_update_headlines_activate), target);

  /*        /SHRINK BUTTON */
  button_shrink = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
					    "gtk-zoom-out",
					    _("Shrink"),
					    NULL, NULL, NULL, -1);
  gtk_widget_show (button_shrink);
  gtk_signal_connect (GTK_OBJECT (button_shrink), "clicked",
		      GTK_SIGNAL_FUNC (on_shrink_activate), target);

  /*        /EXPAND BUTTON */
  button_expand = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
					    "gtk-zoom-in",
					    _("Expand"),
					    NULL, NULL, NULL, -1);
  gtk_widget_show (button_expand);
  gtk_signal_connect (GTK_OBJECT (button_expand), "clicked",
		      GTK_SIGNAL_FUNC (on_expand_activate), target);

  /*        /PREFRENCE BUTTON */
  button_preference = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
						"gtk-preferences",
						_("Preference"),
						NULL, NULL, NULL, -1);
  gtk_widget_show (button_preference);
  gtk_signal_connect (GTK_OBJECT (button_preference), "clicked",
		      GTK_SIGNAL_FUNC (on_preferences_activate), NULL);

  /*        /QUIT BUTTON */
  button_quit = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
					  "gtk-quit",
					  _("Quit"),
					  NULL, NULL, NULL, -1);
  gtk_widget_show (button_quit);
  gtk_signal_connect (GTK_OBJECT (button_quit), "clicked",
		      GTK_SIGNAL_FUNC (on_quit_activate), NULL);


  /* APPBAR */
  appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  gtk_widget_show (appbar);
  gnome_app_set_statusbar (GNOME_APP (app), appbar);

  gnome_app_install_menu_hints (GNOME_APP (app), menubar_uiinfo);

  /* VBOX for CONTENTS */
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gnome_app_set_contents (GNOME_APP (app), vbox);

  /* MAIN CONTENTS */
  while (ptarget->url) {
    gchar *p, button_label[MAXBUFF_LABEL];

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    ptarget->entry = gtk_entry_new ();
    gtk_widget_show (ptarget->entry);
    gtk_box_pack_start (GTK_BOX (hbox), ptarget->entry, TRUE, TRUE, 0);
    gtk_entry_set_editable (GTK_ENTRY (ptarget->entry), FALSE);
    gtk_widget_set_usize (ptarget->entry, ENTRY_WIDGET_USIZE, 0);

    strcpy (button_label, ptarget->url + 7);
    if ((p = strstr (button_label, "/"))) *p = '\0';
    ptarget->url_button = gtk_button_new_with_label (button_label);
    gtk_widget_show (ptarget->url_button);
    gtk_box_pack_start (GTK_BOX (hbox), ptarget->url_button, FALSE, FALSE, 0);
    gtk_widget_set_usize (ptarget->url_button, LABEL_WIDGET_USIZE, 0);

    gtk_signal_connect (GTK_OBJECT (ptarget->url_button), "clicked",
			GTK_SIGNAL_FUNC (on_url_button_clicked),
			ptarget->url);
    gtk_signal_connect (GTK_OBJECT (ptarget->url_button), "enter",
			GTK_SIGNAL_FUNC (on_url_button_enter),
			ptarget->url);
    gtk_signal_connect (GTK_OBJECT (ptarget->url_button), "leave",
			GTK_SIGNAL_FUNC (on_url_button_leave),
			ptarget->url);

    ptarget++;
  }

  /* DELETE EVENT CALLBACK */
  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
		      GTK_SIGNAL_FUNC (on_quit_activate), NULL);

  /* WINDOW ICON */
  gtk_signal_connect (GTK_OBJECT (app), "realize",
		      GTK_SIGNAL_FUNC (set_window_icon), NULL);
  return app;
}

void
put_status_info (gpointer form, gchar *item)
{
  static gchar status_info_buffer[MAXBUFF_LABEL];

  if (!strcmp ((gchar *) form, "(status clear)")) {
    gnome_appbar_clear_stack (GNOME_APPBAR (appbar));
  } else {
    sprintf (status_info_buffer, form, item);
    gnome_appbar_push (GNOME_APPBAR (appbar), status_info_buffer);
    while (gtk_events_pending ())
      gtk_main_iteration ();
  }
}

void
flash_status_info (gpointer form, gchar *item)
{
  static gchar status_info_buffer[MAXBUFF_LABEL];

  sprintf (status_info_buffer, form, item);
  gnome_app_flash (GNOME_APP (app), status_info_buffer);
}

void
main_window_control (gchar * control)
{
  if (!strcmp (control, "HIDE"))
    gtk_widget_hide (app);
  else if (!strcmp (control, "SHOW"))
    gtk_widget_show (app);
}

void
auto_update (gboolean state, gint interval)
{
  static gint ptimer = 0;

  if ((state == TRUE) && (ptimer == 0)) {
    ptimer = gtk_timeout_add (60 * 1000 * interval,
			      (GtkFunction) update_headlines, target);
  } else if ((state == FALSE) && (ptimer != 0)) {
    gtk_timeout_remove (ptimer);
    ptimer = 0;
  }
}

JhPreference *
getaddr_pref (void)
{
  return &pref;
}
