#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

#include "jheadlinestypes.h"
#include "network.h"
#include "interface.h"

static gchar *
kanji_code_filter (gchar * src)
{
  static gchar buff[MAXBUFF_LINE];
  static GString *tmp_in, *tmp_out;
  static gboolean filename_allocated = FALSE;
  FILE *fp;
  gchar command[256];

  if (!filename_allocated) {
    tmp_in  = g_string_new (g_getenv ("HOME"));
    tmp_out = g_string_new (g_getenv ("HOME"));
    g_string_append (tmp_in,  "/.jheadlines/");
    g_string_append (tmp_out, "/.jheadlines/");
    mkdir (tmp_in->str, 0775 & ~umask (0));
    g_string_append (tmp_in,  "tmp_in");
    g_string_append (tmp_out, "tmp_out");
    filename_allocated = TRUE;
  }

  if ((fp = fopen (tmp_in->str, "w")) == NULL) {
    g_warning (_("Failed to open(w) %s for kanji conversion."), tmp_in->str);
    return src;
  }
  fputs (src, fp);
  fclose (fp);

  sprintf (command, "nkf -w %s > %s", tmp_in->str, tmp_out->str);
  system (command);

  if ((fp = fopen (tmp_out->str, "r")) == NULL) {
    g_warning (_("Failed to open(r) %s for kanji conversion."), tmp_out->str);
    return src;
  }

  fgets (buff, MAXBUFF_LINE, fp);
  fclose (fp);

  return buff;
}

static gchar *
strip_html_tags (gchar * org)
{
  static gchar body[MAXBUFF_LINE];

  gboolean in_tag = FALSE;
  gboolean in_white = TRUE;
  gint i, iout = 0;

  for (i = 0; i < strlen (org); i++) {
    if (in_white) {
      if (org[i] == ' ' || org[i] == '\t')
	continue;
      else
	in_white = FALSE;
    }

    if (org[i] == ' ' || org[i] == '\t')
      in_white = TRUE;

    if (!strncmp (org + i, "&nbsp", 5)) {
      i += 5;
      continue;
    }

    if (in_tag) {
      if (org[i] == '>') {
	in_tag = FALSE;
	continue;
      }
    } else {
      if (org[i] == '<') {
	in_tag = TRUE;
	continue;
      } else {
	body[iout++] = org[i];
      }
    }
  }

  body[iout] = '\0';

  return body;
}

static gchar *
grep_headlines (FILE * fp, gchar * url)
{
  gchar buff[MAXBUFF_LINE];
  gchar *headline;

  if (!strncmp (url, "http://www.asahi.com/", 21)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "HeadlineTop"))
	break;
    fgets (buff, sizeof (buff), fp);
    
  } else if (!strncmp (url, "http://mainichi.jp/", 19)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "headline-top"))
	break;
    fgets (buff, sizeof (buff), fp);
    fgets (buff, sizeof (buff), fp);

  } else if (!strncmp (url, "http://www.nikkei.com/", 22)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "<h3>"))
	break;
    fgets (buff, sizeof (buff), fp);

  } else if (!strncmp (url, "http://www.yomiuri.co.jp/", 25)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "headline"))
	break;

  } else if (!strncmp (url, "http://www.sankei.com/", 22)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "<h1>"))
	break;

  } else if (!strncmp (url, "http://www.tokyo-np.co.jp/", 26)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "head1"))
	break;

  } else if (!strncmp (url, "http://www.jiji.com/", 20)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "TopicsList"))
	break;
    fgets (buff, sizeof (buff), fp);
    fgets (buff, sizeof (buff), fp);

  } else if (!strncmp (url, "http://www.cnn.co.jp/", 21)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "topStories"))
	break;

  } else if (!strncmp (url, "http://news.livedoor.com/", 25)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "topicsList"))
	break;
    fgets (buff, sizeof (buff), fp);

  } else if (!strncmp (url, "http://www.nikkansports.com/", 28)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "h1 class=\"top-news_heading\""))
	break;

  } else if (!strncmp (url, "http://rocketnews24.com/", 24)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "<h2 class="))
	break;

  } else if (!strncmp (url, "http://www.itmedia.co.jp/news/", 30)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "<h3>"))
	break;

  } else if (!strncmp (url, "http://ascii.jp/", 16)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "<h2>"))
	break;

  } else if (!strncmp (url, "http://japan.cnet.com/", 22)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "headline"))
	break;
    fgets (buff, sizeof (buff), fp);

  } else if (!strncmp (url, "http://www.itmedia.co.jp/", 25)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "<h5>"))
	break;

  } else if (!strncmp (url, "http://jp.reuters.com/", 22)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "h2 class=\"story-title\""))
	break;
    fgets (buff, sizeof (buff), fp);

  } else if (!strncmp (url, "http://www.j-cast.com/", 22)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "h3 class=\"entry-title\""))
	break;

  } else if (!strncmp (url, "http://www.daily.co.jp/", 22)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "article"))
	break;
    fgets (buff, sizeof (buff), fp);

  } else if (!strncmp (url, "http://news.yahoo.co.jp/", 24)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "h1 class=\"ttl\""))
	break;

  } else if (!strncmp (url, "http://news.infoseek.co.jp/", 27)) {
    while (fgets (buff, sizeof (buff), fp))
      if (strstr (buff, "TopTopicsTitle_all"))
	break;

  } else {
    strcpy (buff, _("ERROR: No algorithm for select headline"));
  }

  headline = kanji_code_filter (buff);
  headline = strip_html_tags (headline);

  return headline;
}

gchar *
get_headline (JhTarget *target)
{
  static gchar buff[MAXBUFF_LINE];

  FILE *fp;
  gint socket_id;
  struct sockaddr_in server;
  struct hostent *servhost;
  struct servent *service;
  gchar host[MAXBUFF_LABEL], path[MAXBUFF_LABEL], host_path[MAXBUFF_LABEL];
  gchar *headline = NULL;
  gushort port = 80;
  gboolean modified, has_lastmod_header;

FUNCTION_START:


  if ((strstr (target->url, "http://")              &&
       sscanf (target->url, "http://%s", host_path) &&
       strcmp (target->url, "http://"))) {

    char *p;

    p = strchr (host_path, '/');
    if (p != NULL) {
      strcpy (path, p);
      *p = '\0';
      strcpy (host, host_path);
    } else {
      strcpy (path, "/");
      strcpy (host, host_path);
    }

    p = strchr (host, ':');
    if (p != NULL) {
      port = atoi (p + 1);
      if (port <= 0) {
	port = 80;
      }
      *p = '\0';
    }
  } else {
    put_status_info (_("FAILED to analize URL: %s"), target->url);
    g_warning ("FAILED to analize URL: %s", target->url);
    return NULL;
  }

  if ((servhost = gethostbyname (host)) == NULL) {
    put_status_info (_("FAILED to get IP-address: %s"), host);
    return NULL;
  } else
    put_status_info (_("accessing: %s"), host);

  bzero ((char *) &server, sizeof (server));
  server.sin_family = AF_INET;
  bcopy (servhost->h_addr, (char *) &server.sin_addr, servhost->h_length);

  if (port != 0) {
    server.sin_port = htons (port);
  } else {
    service = getservbyname ("http", "tcp");
    if (service != NULL) {
      server.sin_port = service->s_port;
    } else {
      server.sin_port = htons (80);
    }
  }

  if ((socket_id = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
    put_status_info (_("FAILED to create socket: %s"), host);
    return NULL;
  } else
    put_status_info (_("created socket success: %s"), host);

  if (connect (socket_id, (struct sockaddr *) &server, sizeof (server)) == -1) {
    put_status_info (_("FAILED to open connection"), NULL);
    return NULL;
  } else
    put_status_info (_("opening connection: %s"), host);

  if ((fp = fdopen (socket_id, "r+")) == NULL) {
    put_status_info (_("FAILED to fdopen: %s"), host);
    return NULL;
  } else
    put_status_info (_("fdopen success: %s"), host);

  setvbuf (fp, NULL, _IONBF, 0);
  put_status_info (_("sending http GET request: %s"), host);

  /* GET method */
  fprintf (fp, "GET %s HTTP/1.0\r\n", path);
  fprintf (fp, "Host: %s:%d\r\n", host, port);
  fprintf (fp, "User-Agent: jheadlines/0.1\r\n");
  fprintf (fp, "\r\n");

  /* Header part */
  put_status_info (_("receiving header part: %s"), host);

  modified = FALSE;
  has_lastmod_header = FALSE;

  while (fgets (buff, sizeof (buff), fp)) {
    /*
    if (strstr (buff, "Location:")) {
      gchar tmp[MAXBUFF_LINE], *p;
      //	     for example...
      //
      //	     HTTP/1.0 302 Moved Temporarily
      //	     Date: Sat, 21 Dec 2002 15:06:47 GMT
      //	     Server: Zeus/4.1
      //	     Location: /weekend/index.html
      //	     Via: 1.1 moschino (NetCache NetApp/5.2.1R2)
      //
      //	     g_message ("FOUND Loacation, %s", target->url);
      fclose (fp);
      close (socket_id);

      g_printf ("Location");
      g_printf ("  old: %s", target->url);

      if (!strncmp (buff, "Location: http://", 15)) {
	p = strchr (buff, ':');
	target->url = g_strdup (p + 2);
      } else {
	strcpy (tmp, target->url);
	p = strchr (buff, '/');
	strcat (tmp, ++p);
	target->url = g_strdup (tmp);
      }
      g_printf ("  new: %s", target->url);
	
      fprintf (stderr, "POINT-2 <%s>\n", target->url);

      goto FUNCTION_START;

    } else if (!strncmp (buff, "Last-Modified", 13)) {
    */

    if (!strncmp (buff, "Last-Modified", 13)) {
      has_lastmod_header = TRUE;
      if (target->last_modified == NULL) {
	modified = TRUE;
	target->last_modified = g_strdup (buff);
      } else {
	if (strcmp (buff, target->last_modified)) {
	  g_free (target->last_modified);
	  modified = TRUE;
	  target->last_modified = g_strdup (buff);
	}
      }
    }

    if (buff[0] == '\r' && buff[1] == '\n')
      break;
  }
  if (has_lastmod_header && !modified) {
    headline = "(no change)";
  } else {				/* search headlines in html document */

    put_status_info (_("receiving body part: %s"), host);
    headline = grep_headlines (fp, target->url);
  }

  fclose (fp);
  close (socket_id);

  return headline;
}
