#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "jheadlinestypes.h"
#include "shrink.h"
#include "interface.h"

static GtkWidget *swin, *label;
static gint ptimer = 0;

static void
destroy_swin (void)
{
  if (ptimer)
    {
      gtk_timeout_remove (ptimer);
      ptimer = 0;
    }

  gtk_widget_destroy (swin);
  main_window_control ("SHOW");
}

static gboolean
swin_scroll_message (JhTarget *target)
{
  static gint current = 0;

  gchar buff[MAXBUFF_LINE], address[MAXBUFF_LABEL];
  gchar *p;

  if (!(target + current)->url)
    current = 0;

  strcpy (address, (target + current)->url + 7);
  if ((p = strstr (address, "/")))
    *p = '\0';
  sprintf (buff, "%s: %s", address,
	   gtk_entry_get_text (GTK_ENTRY ((target + current)->entry)));
  gtk_label_set_text (GTK_LABEL (label), buff);
  gtk_window_set_title (GTK_WINDOW (swin), buff + strlen (address) + 1);

  while (gtk_events_pending ())
    gtk_main_iteration ();

  current++;

  return TRUE;
}

static void
swin_key_press (GtkWidget *widget, GdkEventKey *event, JhTarget *target)
{
  switch (event->keyval)
    {
    case GDK_Escape:
    case GDK_q:
      destroy_swin ();
      break;

    case GDK_space:
    case GDK_Tab:
      swin_scroll_message (target);
      break;
    }
}

void
on_shrink_activate (GtkWidget *widget, JhTarget *target)
{
  JhPreference *pref = getaddr_pref ();

  main_window_control ("HIDE");

  /* swin: shrinked window */
  swin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (swin), _("jheadlines"));
  gtk_window_set_policy (GTK_WINDOW (swin), TRUE, TRUE, TRUE);
  gtk_container_set_border_width (GTK_CONTAINER (swin), 5);
  gtk_window_set_position (GTK_WINDOW (swin), GTK_WIN_POS_MOUSE);

  /* label: for headline display */
  label = gtk_label_new (_("jheadlines: News-headline from Web"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (swin), "label", label,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_set_usize (label, 640, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
  gtk_container_add (GTK_CONTAINER (swin), label);

  gtk_signal_connect (GTK_OBJECT (swin), "delete_event",
		      GTK_SIGNAL_FUNC (destroy_swin), NULL);
  gtk_signal_connect (GTK_OBJECT (swin), "destroy",
		      GTK_SIGNAL_FUNC (destroy_swin), NULL);
  gtk_signal_connect (GTK_OBJECT (swin), "key_press_event",
		      GTK_SIGNAL_FUNC (swin_key_press), target);
  gtk_signal_connect (GTK_OBJECT (swin), "realize",
		      GTK_SIGNAL_FUNC (set_window_icon), NULL);
  gtk_widget_set_events (swin, GDK_KEY_PRESS_MASK);

  gtk_widget_show_all (swin);

  ptimer = gtk_timeout_add (1000 * pref->shrink_message_interval,
			    (GtkFunction) swin_scroll_message, target);
}
